@@ -25,6 +25,16 @@ from saltfactories.utils import processes
 
 
 class SaltFactoriesManager(object):
+    """
+    The :class:`SaltFactoriesManager` is responsible for configuring and spawning Salt Daemons and
+    making sure that any salt CLI tools are "targetted" to the right daemon.
+
+    It also keeps track of which daemons were started and adds their termination routines to PyTest's
+    request finalization routines.
+
+    If process statistics are enabled, it also adds the started daemons to those statistics.
+    """
+
     def __init__(
         self,
         pytestconfig,
@@ -70,6 +80,9 @@ class SaltFactoriesManager(object):
 
     @staticmethod
     def get_running_username():
+        """
+        Returns the current username
+        """
         try:
             return SaltFactoriesManager.get_running_username.__username__
         except AttributeError:
@@ -87,10 +100,16 @@ class SaltFactoriesManager(object):
 
     @staticmethod
     def get_salt_log_handlers_path():
+        """
+        Returns the path to the Salt log handler this plugin provides
+        """
         return os.path.join(saltfactories.CODE_ROOT_DIR, "utils", "salt", "log_handlers")
 
     @staticmethod
     def get_salt_engines_path():
+        """
+        Returns the path to the Salt engine this plugin provides
+        """
         return os.path.join(saltfactories.CODE_ROOT_DIR, "utils", "salt", "engines")
 
     def final_minion_config_tweaks(self, config):
@@ -138,24 +157,34 @@ class SaltFactoriesManager(object):
         log_config["level"] = self.log_server_level
 
     def configure_minion(self, request, minion_id, master_id=None):
+        """
+        Configure a salt-minion
+        """
         if minion_id in self.cache["configs"]["minions"]:
             return self.cache["configs"]["minions"][minion_id]
 
+        root_dir = self._get_root_dir_for_daemon(minion_id)
+
         master_port = None
         if master_id is not None:
             master_config = self.cache["configs"]["masters"][master_id]
             master_port = master_config.get("ret_port")
         default_options = self.pytestconfig.hook.pytest_saltfactories_generate_default_minion_configuration(
-            request=request, factories_manager=self, minion_id=minion_id, master_port=master_port
+            request=request,
+            factories_manager=self,
+            root_dir=root_dir,
+            minion_id=minion_id,
+            master_port=master_port,
         )
         config_overrides = self.pytestconfig.hook.pytest_saltfactories_minion_configuration_overrides(
             request=request,
             factories_manager=self,
+            root_dir=root_dir,
             minion_id=minion_id,
             default_options=default_options,
         )
         minion_config = minion.MinionFactory.default_config(
-            self.root_dir,
+            root_dir,
             minion_id=minion_id,
             default_options=default_options,
             config_overrides=config_overrides,
@@ -182,6 +211,9 @@ class SaltFactoriesManager(object):
         return minion_config
 
     def spawn_minion(self, request, minion_id, master_id=None):
+        """
+        Spawn a salt-minion
+        """
         if minion_id in self.cache["minions"]:
             raise RuntimeError("A minion by the ID of '{}' was already spawned".format(minion_id))
 
@@ -189,45 +221,30 @@ class SaltFactoriesManager(object):
         if minion_config is None:
             minion_config = self.configure_minion(request, minion_id, master_id=master_id)
 
-        script_path = cli_scripts.generate_script(
-            self.scripts_dir,
-            "salt-minion",
-            executable=self.executable,
-            code_dir=self.code_dir,
-            inject_coverage=self.inject_coverage,
-            inject_sitecustomize=self.inject_sitecustomize,
+        return self._start_daemon(
+            request, "salt-minion", minion_config, processes.SaltMinion, "minions", minion_id
         )
-        proc = processes.start_daemon(
-            minion_config,
-            script_path,
-            processes.SaltMinion,
-            # start_timeout=self.start_timeout,
-            start_timeout=20,
-            slow_stop=self.slow_stop,
-            environ=self.environ,
-            cwd=self.cwd,
-            max_attempts=3,
-            event_listener=self.event_listener,
-        )
-        self.cache["minions"][minion_id] = proc
-        self.track_process_stats(proc)
-        request.addfinalizer(proc.terminate)
-        request.addfinalizer(lambda: self.cache["minions"].pop(minion_id))
-        return proc
 
     def configure_master(self, request, master_id, order_masters=False, master_of_masters_id=None):
+        """
+        Configure a salt-master
+        """
         if master_id in self.cache["configs"]["masters"]:
             return self.cache["configs"]["masters"][master_id]
 
+        root_dir = self._get_root_dir_for_daemon(master_id)
+
         default_options = self.pytestconfig.hook.pytest_saltfactories_generate_default_master_configuration(
             request=request,
             factories_manager=self,
+            root_dir=root_dir,
             master_id=master_id,
             order_masters=order_masters,
         )
         config_overrides = self.pytestconfig.hook.pytest_saltfactories_master_configuration_overrides(
             request=request,
             factories_manager=self,
+            root_dir=root_dir,
             master_id=master_id,
             default_options=default_options,
             order_masters=order_masters,
@@ -243,7 +260,7 @@ class SaltFactoriesManager(object):
             config_overrides["syndic_master_port"] = master_of_masters_config["ret_port"]
 
         master_config = master.MasterFactory.default_config(
-            self.root_dir,
+            root_dir,
             master_id=master_id,
             default_options=default_options,
             config_overrides=config_overrides,
@@ -265,6 +282,9 @@ class SaltFactoriesManager(object):
     def spawn_master(
         self, request, master_id, order_masters=False, master_of_masters_id=None,
     ):
+        """
+        Spawn a salt-master
+        """
         if master_id in self.cache["masters"]:
             raise RuntimeError("A master by the ID of '{}' was already spawned".format(master_id))
 
@@ -277,33 +297,14 @@ class SaltFactoriesManager(object):
                 master_of_masters_id=master_of_masters_id,
             )
 
-        script_path = cli_scripts.generate_script(
-            self.scripts_dir,
-            "salt-master",
-            executable=self.executable,
-            code_dir=self.code_dir,
-            inject_coverage=self.inject_coverage,
-            inject_sitecustomize=self.inject_sitecustomize,
+        return self._start_daemon(
+            request, "salt-master", master_config, processes.SaltMaster, "masters", master_id
         )
-        proc = processes.start_daemon(
-            master_config,
-            script_path,
-            processes.SaltMaster,
-            start_timeout=self.start_timeout,
-            slow_stop=self.slow_stop,
-            environ=self.environ,
-            cwd=self.cwd,
-            max_attempts=3,
-            event_listener=self.event_listener,
-        )
-        self.cache["masters"][master_id] = proc
-        self.track_process_stats(proc)
-        request.addfinalizer(proc.terminate)
-        request.addfinalizer(lambda: self.cache["masters"].pop(master_id))
-        return proc
 
     def configure_syndic(self, request, syndic_id, master_of_masters_id=None):
         """
+        Configure a salt-syndic.
+
         In order for the syndic to be reactive, it actually needs three(3) daemons running, `salt-master`,
         `salt-minion` and `salt-syndic`.
 
@@ -327,9 +328,12 @@ class SaltFactoriesManager(object):
 
         syndic_master_port = master_of_masters_config["ret_port"]
 
+        root_dir = self._get_root_dir_for_daemon(syndic_id)
+
         default_options = self.pytestconfig.hook.pytest_saltfactories_generate_default_syndic_configuration(
             request=request,
             factories_manager=self,
+            root_dir=root_dir,
             syndic_id=syndic_id,
             syndic_master_port=syndic_master_port,
         )
@@ -337,12 +341,13 @@ class SaltFactoriesManager(object):
         config_overrides = self.pytestconfig.hook.pytest_saltfactories_syndic_configuration_overrides(
             request=request,
             factories_manager=self,
+            root_dir=root_dir,
             syndic_id=syndic_id,
             default_options=default_options,
         )
 
         syndic_setup_config = syndic.SyndicFactory.default_config(
-            self.root_dir,
+            root_dir,
             syndic_id=syndic_id,
             default_options=default_options,
             config_overrides=config_overrides,
@@ -395,6 +400,9 @@ class SaltFactoriesManager(object):
         return syndic_config
 
     def spawn_syndic(self, request, syndic_id, master_of_masters_id=None):
+        """
+        Spawn a salt-syndic
+        """
         if syndic_id in self.cache["syndics"]:
             raise RuntimeError("A syndic by the ID of '{}' was already spawned".format(syndic_id))
 
@@ -411,32 +419,14 @@ class SaltFactoriesManager(object):
         if syndic_id not in self.cache["minions"]:
             self.spawn_minion(request, syndic_id)
 
-        script_path = cli_scripts.generate_script(
-            self.scripts_dir,
-            "salt-syndic",
-            executable=self.executable,
-            code_dir=self.code_dir,
-            inject_coverage=self.inject_coverage,
-            inject_sitecustomize=self.inject_sitecustomize,
+        return self._start_daemon(
+            request, "salt-syndic", syndic_config, processes.SaltSyndic, "syndics", syndic_id
         )
-        proc = processes.start_daemon(
-            syndic_config,
-            script_path,
-            processes.SaltSyndic,
-            start_timeout=self.start_timeout,
-            slow_stop=self.slow_stop,
-            environ=self.environ,
-            cwd=self.cwd,
-            max_attempts=3,
-            event_listener=self.event_listener,
-        )
-        self.cache["syndics"][syndic_id] = proc
-        self.track_process_stats(proc)
-        request.addfinalizer(proc.terminate)
-        request.addfinalizer(lambda: self.cache["syndics"].pop(syndic_id))
-        return proc
 
     def configure_proxy_minion(self, request, proxy_minion_id, master_id=None):
+        """
+        Configure a salt-proxy
+        """
         if proxy_minion_id in self.cache["configs"]["proxy_minions"]:
             return self.cache["configs"]["proxy_minions"][proxy_minion_id]
 
@@ -444,20 +434,25 @@ class SaltFactoriesManager(object):
         if master_id is not None:
             master_config = self.cache["configs"]["masters"][master_id]
             master_port = master_config.get("ret_port")
+
+        root_dir = self._get_root_dir_for_daemon(proxy_minion_id)
+
         default_options = self.pytestconfig.hook.pytest_saltfactories_generate_default_proxy_minion_configuration(
             request=request,
             factories_manager=self,
+            root_dir=root_dir,
             proxy_minion_id=proxy_minion_id,
             master_port=master_port,
         )
         config_overrides = self.pytestconfig.hook.pytest_saltfactories_proxy_minion_configuration_overrides(
             request=request,
             factories_manager=self,
+            root_dir=root_dir,
             proxy_minion_id=proxy_minion_id,
             default_options=default_options,
         )
         proxy_minion_config = proxy.ProxyMinionFactory.default_config(
-            self.root_dir,
+            root_dir,
             proxy_minion_id=proxy_minion_id,
             default_options=default_options,
             config_overrides=config_overrides,
@@ -484,6 +479,9 @@ class SaltFactoriesManager(object):
         return proxy_minion_config
 
     def spawn_proxy_minion(self, request, proxy_minion_id, master_id=None):
+        """
+        Spawn a salt-proxy
+        """
         if proxy_minion_id in self.cache["proxy_minions"]:
             raise RuntimeError(
                 "A proxy_minion by the ID of '{}' was already spawned".format(proxy_minion_id)
@@ -495,33 +493,19 @@ class SaltFactoriesManager(object):
                 request, proxy_minion_id, master_id=master_id
             )
 
-        script_path = cli_scripts.generate_script(
-            self.scripts_dir,
+        return self._start_daemon(
+            request,
             "salt-proxy",
-            executable=self.executable,
-            code_dir=self.code_dir,
-            inject_coverage=self.inject_coverage,
-            inject_sitecustomize=self.inject_sitecustomize,
-        )
-        proc = processes.start_daemon(
             proxy_minion_config,
-            script_path,
             processes.SaltProxyMinion,
-            # start_timeout=self.start_timeout,
-            start_timeout=20,
-            slow_stop=self.slow_stop,
-            environ=self.environ,
-            cwd=self.cwd,
-            max_attempts=3,
-            event_listener=self.event_listener,
+            "proxy_minions",
+            proxy_minion_id,
         )
-        self.cache["proxy_minions"][proxy_minion_id] = proc
-        self.track_process_stats(proc)
-        request.addfinalizer(proc.terminate)
-        request.addfinalizer(lambda: self.cache["proxy_minions"].pop(proxy_minion_id))
-        return proc
 
     def get_salt_cli(self, request, master_id):
+        """
+        Return a `salt` CLI process
+        """
         script_path = cli_scripts.generate_script(
             self.scripts_dir,
             "salt",
@@ -533,6 +517,9 @@ class SaltFactoriesManager(object):
         return processes.SaltCLI(script_path, config=self.cache["masters"][master_id].config)
 
     def get_salt_call_cli(self, request, minion_id):
+        """
+        Return a `salt-call` CLI process
+        """
         script_path = cli_scripts.generate_script(
             self.scripts_dir,
             "salt-call",
@@ -558,6 +545,9 @@ class SaltFactoriesManager(object):
                 )
 
     def get_salt_run(self, request, master_id):
+        """
+        Return a `salt-run` CLI process
+        """
         script_path = cli_scripts.generate_script(
             self.scripts_dir,
             "salt-run",
@@ -569,6 +559,9 @@ class SaltFactoriesManager(object):
         return processes.SaltRunCLI(script_path, config=self.cache["masters"][master_id].config)
 
     def get_salt_cp(self, request, master_id):
+        """
+        Return a `salt-cp` CLI process
+        """
         script_path = cli_scripts.generate_script(
             self.scripts_dir,
             "salt-cp",
@@ -580,6 +573,9 @@ class SaltFactoriesManager(object):
         return processes.SaltCpCLI(script_path, config=self.cache["masters"][master_id].config)
 
     def get_salt_key(self, request, master_id):
+        """
+        Return a `salt-key` CLI process
+        """
         script_path = cli_scripts.generate_script(
             self.scripts_dir,
             "salt-key",
@@ -590,7 +586,46 @@ class SaltFactoriesManager(object):
         )
         return processes.SaltKeyCLI(script_path, config=self.cache["masters"][master_id].config)
 
-    def track_process_stats(self, proc):
+    def _start_daemon(
+        self, request, script_name, daemon_config, daemon_class, cache_key, daemon_id
+    ):
+        """
+        Helper method to start daemons
+        """
+        script_path = cli_scripts.generate_script(
+            self.scripts_dir,
+            script_name,
+            executable=self.executable,
+            code_dir=self.code_dir,
+            inject_coverage=self.inject_coverage,
+            inject_sitecustomize=self.inject_sitecustomize,
+        )
+        proc = processes.start_daemon(
+            daemon_config,
+            script_path,
+            daemon_class,
+            start_timeout=self.start_timeout,
+            slow_stop=self.slow_stop,
+            environ=self.environ,
+            cwd=self.cwd,
+            max_attempts=3,
+            event_listener=self.event_listener,
+        )
+        self.cache[cache_key][daemon_id] = proc
         if self.stats_processes:
             process_name = proc.log_prefix.strip().lstrip("[").rstrip("]")
             self.stats_processes[process_name] = psutil.Process(proc.pid)
+        request.addfinalizer(proc.terminate)
+        request.addfinalizer(lambda: self.cache[cache_key].pop(daemon_id))
+        return proc
+
+    def _get_root_dir_for_daemon(self, daemon_id):
+        counter = 1
+        root_dir = self.root_dir.join(daemon_id)
+        while True:
+            if not root_dir.check(dir=True):
+                break
+            root_dir = self.root_dir.join("{}_{}".format(daemon_id, counter))
+            counter += 1
+        root_dir.ensure(dir=True)
+        return root_dir
