@@ -224,6 +224,7 @@ def start_daemon(
     cwd=None,
     max_attempts=3,
     event_listener=None,
+    **extra_daemon_class_kwargs
 ):
     """
     Returns a running process daemon
@@ -243,6 +244,7 @@ def start_daemon(
             slow_stop=slow_stop,
             environ=environ,
             cwd=cwd,
+            **extra_daemon_class_kwargs
         )
         log_prefix = process.log_prefix
         log.info("%sStarting %r. Attempt: %s", log_prefix, process, attempts)
@@ -250,8 +252,15 @@ def start_daemon(
         process.start()
         if process.is_alive():
             try:
-                check_ports = process.get_check_ports()
-                check_events = list(process.get_check_events())
+                try:
+                    check_ports = process.get_check_ports()
+                except AttributeError:
+                    check_ports = False
+
+                try:
+                    check_events = list(process.get_check_events())
+                except AttributeError:
+                    check_events = False
 
                 if not check_ports and not check_events:
                     connectable = True
@@ -451,6 +460,7 @@ class FactoryProcess(object):
         self.environ.setdefault(str("PYTHONDONTWRITEBYTECODE"), str("1"))
         self.cwd = cwd or os.getcwd()
         self._terminal = None
+        self._terminal_result = None
         self._children = []
         self._base_script_args = base_script_args
 
@@ -510,7 +520,7 @@ class FactoryProcess(object):
         Terminate the started daemon
         """
         if self._terminal is None:
-            return
+            return self._terminal_result
         # Allow some time to get all output from process
         time.sleep(0.125)
         log.info("%sStopping %s", self.log_prefix, self.__class__.__name__)
@@ -547,7 +557,8 @@ class FactoryProcess(object):
                     )
                 log_message += "\n"
             log.info(log_message)
-            return ProcessResult(self._terminal.returncode, stdout, stderr)
+            self._terminal_result = ProcessResult(self._terminal.returncode, stdout, stderr)
+            return self._terminal_result
         finally:
             self._terminal = None
             self._children = []
@@ -693,7 +704,7 @@ class SaltScriptBase(FactoryPythonScriptBase, SaltConfigMixin):
         script_args = super(SaltScriptBase, self).get_base_script_args()
         config_dir = self.config_dir
         if config_dir:
-            script_args.extend(["-c", config_dir])
+            script_args.append("--config-dir={}".format(config_dir))
         script_args.append("--log-level=quiet")
         script_args.append("--out=json")
         return script_args
@@ -725,7 +736,7 @@ class SaltDaemonScriptBase(FactoryDaemonScriptBase, FactoryPythonScriptBase, Sal
         script_args = super(SaltDaemonScriptBase, self).get_base_script_args()
         config_dir = self.config_dir
         if config_dir:
-            script_args.extend(["-c", config_dir])
+            script_args.append("--config-dir={}".format(config_dir))
         script_args.append("--log-level=quiet")
         return script_args
 
